/*■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
 Low-cost UECS CO2 fertilizer control node
 (Suit for both sources of Lquid CO2 and burning fossil fuel)
 LU-CO2Fert By T.Hoshi, Kindai Univ.
 2018.12.21 version 1.0
 2021.11.04 ver. 2.0 lower control setpoint is adjustable.
 2022.11.01 ver. 3.0 % of vent. open is also used to 
 change the CO2 control setpoint.
■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■*/
#define verNO   "LU-CO2FRT3" // 10 char.
#define verDATE "20221101" // 8 char.
// このプログラムは、無償提供する代わりに、動作結果に関する
// 一切の保証は致しません。自己責任でお使いください。また、
// 改造も自由ですが、公刊等をされる場合は、「近畿大学　星　
// 岳彦が開発したLU-CO2FRT3プログラムを用いた」と引用して下さい
// ますと、今後の研究開発の励みになりますので、どうぞよろ
// しくお願いします。
/*
-----------------------------------------------------------------------------
LU-CO2FRT3 software are distributed under the 2 clause BSD license.

Copyright 2022, Takehiko Hoshi All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
-----------------------------------------------------------------------------
LU-CO2FRT3ソフトウェアは、2条項BSDライセンスの下に配布されます。

Copyright 2022, Takehiko Hoshi All rights reserved.

  ソースコード形式かバイナリ形式か、変更するかしないかを問わず、以下の条件を
満たす場合に限り、再頒布および使用が許可されます。

1.ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、および下記免責
  条項を含めること。
2.バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の資料に、上記の
  著作権表示、本条件一覧、および下記免責条項を含めること。

  本ソフトウェアは、著作権者およびコントリビューターによって「現状のまま」提供
されており、明示黙示を問わず、商業的な使用可能性、および特定の目的に対する適合
性に関する暗黙の保証も含め、またそれに限定されない、いかなる保証もありません。
著作権者もコントリビューターも、事由のいかんを問わず、 損害発生の原因いかんを
問わず、かつ責任の根拠が契約であるか厳格責任であるか（過失その他の）不法行為で
あるかを問わず、仮にそのような損害が発生する可能性を知らされていたとしても、
本ソフトウェアの使用によって発生した （代替品または代用サービスの調達、使用の
喪失、データの喪失、利益の喪失、業務の中断も含め、またそれに限定されない）直接
損害、間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害について、
一切責任を負わないものとします。
-----------------------------------------------------------------------------
*/
// このシールドはArduino MEGA 2560 + Ethernet Shiedld2の
// 組み合わせで動作させることを前提にしています。
// Ethernet Sheild 2基板のシールのmacアドレスに変更して
// ください。macアドレスとは。90:A2:DA:0F:xx:xx のような8個
// の16進数です。これをばらして下の0xの後に転記してください。
// mac addresses
#define mac0 0x90
#define mac1 0xA2
#define mac2 0xDA
#define mac3 0x10
#define mac4 0xDE
#define mac5 0xDA

// UECS用のライブラリUARDECSのインストール
// https://uecs.org/arduino/uardecs.html が必要です。
#include <Uardecs_mega.h>
#include <SPI.h>
#include <Ethernet2.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <EEPROM.h>

//////////////////////
// 定数関連定義
//////////////////////

//制御出力の定義
const uint8_t CO2out = 0 ; // CO2電磁弁(液化用)
const uint8_t CO2outB = 1 ; // CO2供給(燃焼用)
const uint8_t fanout = 3 ; // 暖房機循環FAN

//
//IPアドレスをリセットするピンの割付
//
//Pin ID. This pin is pull-upped automatically.
const byte U_InitPin = 57 ; // for Arduino MEGA 2560 
//const byte U_InitPin = 17 ; // for Arduino Ethernet
//This shields is assigned as A3 for reset pin.
//When U_InitPin status equals this value,IP address is set "192.168.1.7".
//(This is added Ver3 over)
const byte U_InitPin_Sense= HIGH ;  

/////////////////////////////
// ノードの基礎情報の設定
/////////////////////////////

const char U_name[] PROGMEM = verNO ;
const char U_vender[] PROGMEM = "Hoshi-lab.";
const char U_uecsid[] PROGMEM = "011009001005";
const char U_footnote[] PROGMEM = "Product on UARDECS121 by <A HREF=\"https://hoshi-lab.info/\">Kindai, hoshi-lab.</A>";
char U_nodename[20] = verNO ;
UECSOriginalAttribute U_orgAttribute;

////////////////////////////////////////////
// CO2HEATノードの変数設定とライブラリ
////////////////////////////////////////////

// 入出力ポートの定義
const int port[4] = { 7, 8, 5, 6 } ;
// ウオッチドッグタイマーのリセットポート
const int wdt = 9 ; // 0 = clear timer
// ディジタル出力値の一時記憶装置のトリガポート
const int ratch = 3 ; //  0->1 ratch
// ディジタル入力値のイネーブルポート
const int in_out = 2 ; // in = 1 out =0

//制御関係定数・変数
const int C_CO2presec = 30 ; // CO2の注入開始までの待ち時間(秒)
int CO2workTime ; // 施用していないとき0, 施用開始からの秒数
                  // この値で送風ファンとCO2電磁弁の制御を実施
const int C_minDarkKeepT = 14400 ; // 暗黒がこれ以上の秒数継続したのち明るくなったら
                                   // 夜明けと診断する 14,400秒=4時間
int SrPreCO2T ; // 夜明プリCO2施用継続秒数 0で終了
int CO2_supply, FAN_operate, CO2B_supply ; // 各運転状況フラグ HIGHで運転
int innerTime ; // 内部時刻hhmm
int secCount ; // 60秒に一回の処理

// 作業変数
int i ; // ループ用変数
int DOpin[4] ; // 制御出力配列
int DIpin[4] ; // ディジタル入力配列

// Define variables for Web interface
signed long W_CO2cond ; // CO2制御状態 (生ガス)
signed long W_DCO2supTime ; // 昨日CO2供給時間
signed long W_CO2condB ; // CO2制御状態 (燃焼)
signed long W_DCO2supBTime ; // 昨日CO2供給時間
signed long W_Chmode ; // 自動モード
// 生ガス用設定値
signed long W_CO2highSP ; // 設定CO2濃度
signed long W_CO2injsec ; // CO2注入時間
signed long W_CO2postsec ; // CO2後待時間
// 燃焼用設定値
signed long W_BCO2HhighSP ; // 設定CO2濃度
signed long W_BCO2HlowSP ; // 設定CO2濃度
signed long W_BreburnWT ; // 再燃焼待ち時間
signed long W_BstopT ; // 消火後経過時間
signed long W_BpreCO2Tsr ; // 夜明け時のプリ施用時間
signed long W_darkKeepT ; // 暗黒継続時間　夜明プリ施用用
// 共通設定値
signed long W_CO2LowSP ; // 換気時のCO2施用設定濃度
signed long W_CO2vopUse ; // 換気窓開度0%で高濃度施用設定値を使う
signed long W_CO2tempOpen ; // CO2上限気温
signed long W_CO2tempPhsy ; // CO2下限気温
signed long W_CO2solarPhsy ; // CO2下限日射

// ウオッチドッグタイマーのクリア(2秒以内に1回の実行が必要)
void clearWdt() {
  digitalWrite(wdt, LOW) ;
  delayMicroseconds(100) ;
  digitalWrite(wdt, HIGH) ;
}

// ディジタル値を出力する d[0]-d[3]の値 0はOFF、1はON
void dout(int *d) {
  digitalWrite(in_out, LOW) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], OUTPUT) ;
    digitalWrite(port[i], d[i]) ;
  }
  delayMicroseconds(1) ;
  digitalWrite(ratch, HIGH) ;
  delayMicroseconds(1) ; 
  digitalWrite(ratch, LOW) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], INPUT) ;
  }
  digitalWrite(in_out, HIGH) ;
}

// 制御出力を全てOFFにする
void clearDout() {
  int i;
  for(i=0; i<4; i++) DOpin[i] = LOW ;
  dout(DOpin) ;
  CO2_supply = FAN_operate = CO2B_supply = LOW ;
}

// フラグに従い出力
void controlOut() {
  DOpin[CO2out] = CO2_supply ;
  DOpin[CO2outB] = CO2B_supply ;
  DOpin[fanout] = FAN_operate ;
  dout(DOpin) ;
}
 
// ディジタル入力値の読み込み d[0]-d[3]に値を代入
void din() {
  digitalWrite(in_out, HIGH) ;
  delayMicroseconds(1) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], INPUT) ;
    delayMicroseconds(1) ;
    DIpin[i] = digitalRead(port[i]) ;
  }
}

//////////////////////////////////
// html page setting
//////////////////////////////////

// Define Strings
const char S_CO2cond[] PROGMEM ="液CO2状態" ;
const char S_DCO2supTime[] PROGMEM ="昨日液CO2" ;
const char S_CO2condB[] PROGMEM ="燃CO2状態" ;
const char S_DCO2supBTime[] PROGMEM ="昨日燃CO2" ;
const char S_Chmode[] PROGMEM ="自動モード" ;
const char S_CO2highSP[] PROGMEM ="液CO2濃度" ;
const char S_CO2injsec[] PROGMEM ="液CO2注入" ;
const char S_CO2postsec[] PROGMEM ="液CO2後待" ;
const char S_BCO2HhighSP[] PROGMEM ="燃CO2濃度上" ;
const char S_BCO2HlowSP[] PROGMEM ="燃CO2濃度下" ;
const char S_BreburnWT[] PROGMEM ="再燃待時間" ;
const char S_BstopT[] PROGMEM ="消火後経過" ;
const char S_BpreCO2Tsr[] PROGMEM ="夜明プリ供給" ;
const char S_darkKeepT[] PROGMEM ="暗黒継続" ;
const char S_CO2vopUse[] PROGMEM ="窓開度使用" ;
const char S_CO2tempOpen[] PROGMEM ="上限気温" ;
const char S_CO2tempPhsy[] PROGMEM ="下限気温" ;
const char S_CO2solarPhsy[] PROGMEM ="下限日射" ;
const char S_CO2LowSP[] PROGMEM ="換気CO2濃度" ;

//Select Strings
const char S_nouse[] PROGMEM = "不使用";
const char S_use[] PROGMEM = "使用";
const char S_off[] PROGMEM = "停止";
const char S_fan[] PROGMEM = "送風";
const char S_preCO2[] PROGMEM = "前送風";
const char S_inCO2[] PROGMEM = "供給";
const char S_inSRCO2[] PROGMEM = "夜明供給";
const char S_postCO2[] PROGMEM = "後送風";
const char S_autoCO2[] PROGMEM = "液*CO2";
const char S_autoBCO2[] PROGMEM = "燃#CO2";
const char S_forcefan[] PROGMEM = "強制送風";
const char S_forceCO2[] PROGMEM = "強液CO2";
const char S_forceBCO2[] PROGMEM = "強燃CO2";
const char S_reset[] PROGMEM = "初期化";
const char *SS_CO2vopUse[2]={ S_nouse, S_use };
const char *SS_CO2cond[4]={ S_off, S_preCO2, S_inCO2, S_postCO2 };
const char *SS_BCO2cond[3]={ S_off, S_inCO2, S_inSRCO2 };
const char *SS_Chmode[7]={ S_off, S_autoCO2, S_autoBCO2, S_forcefan, S_forceCO2, S_forceBCO2, S_reset };

//Unit Strings
const char U_sec[] PROGMEM =  "秒";
const char U_temp[] PROGMEM =  "℃";
const char U_time[] PROGMEM =  "時.分";
const char U_flux[] PROGMEM =  "kW/m2";
const char U_ppm[] PROGMEM =  "ppm";

//Detail Strings
const char D_vopUse[] PROGMEM =   "*#窓開度0%で高濃度";
const char D_w30[] PROGMEM =   "*30秒後に";
const char D_wmix[] PROGMEM =   "*混合待ち";
const char D_upLow[] PROGMEM =   "*#以上低濃度(窓開度不使用時)";
const char D_downLow[] PROGMEM =   "*#未満低濃度";
const char D_high[] PROGMEM =   "*高濃度時";
const char D_darkKeep[] PROGMEM =   "#14400<夜明検可";
const char D_Bhigh[] PROGMEM =   "#高濃度上限";
const char D_Blow[] PROGMEM =   "#上>下";
const char D_reburn[] PROGMEM =   "#";
const char D_pre[] PROGMEM =   "#0は停止";
const char D_CO2LowSP[] PROGMEM = "*#低濃度時" ;

// Dummy Strings
const char EMPTY[] PROGMEM= "";
const char** DUMMY = NULL;

const int U_HtmlLine = 19; //Total number of HTML table rows.

struct UECSUserHtml U_html[U_HtmlLine] = {
//{  名前,          入出力形式,     単位,  詳細説明,   選択肢文字列,選択肢数, 値,               最小値,最大値, 小数桁数}
  { S_CO2cond,      UECSSHOWSTRING, EMPTY,  EMPTY,       SS_CO2cond,  4,        &(W_CO2cond),     0,     3,      0},
  { S_DCO2supTime,  UECSSHOWDATA,   U_sec,  EMPTY,       DUMMY,       0,        &(W_DCO2supTime), 0,     0,      0},
  { S_CO2condB,     UECSSHOWSTRING, EMPTY,  EMPTY,       SS_BCO2cond, 2,        &(W_CO2condB),    0,     1,      0},
  { S_DCO2supBTime, UECSSHOWDATA,   U_sec,  EMPTY,       DUMMY,       0,        &(W_DCO2supBTime),0,     0,      0},
  { S_Chmode,       UECSSELECTDATA, EMPTY,  EMPTY,       SS_Chmode,   7,        &(W_Chmode),      0,     6,      0},
  { S_CO2LowSP,     UECSINPUTDATA,  U_ppm,  D_CO2LowSP,  DUMMY,       0,        &(W_CO2LowSP),    100,   2000,   0},
  { S_CO2highSP,    UECSINPUTDATA,  U_ppm,  D_high,      DUMMY,       0,        &(W_CO2highSP),   401,   2000,   0},
  { S_CO2injsec,    UECSINPUTDATA,  U_sec,  D_w30,       DUMMY,       0,        &(W_CO2injsec),   0,     100,    0},
  { S_CO2postsec,   UECSINPUTDATA,  U_sec,  D_wmix,      DUMMY,       0,        &(W_CO2postsec),  0,     1800,   0},
  { S_BCO2HhighSP,  UECSINPUTDATA,  U_ppm,  D_Bhigh,     DUMMY,       0,        &(W_BCO2HhighSP), 401,   2000,   0},
  { S_BCO2HlowSP,   UECSINPUTDATA,  U_ppm,  D_Blow,      DUMMY,       0,        &(W_BCO2HlowSP),  401,   2000,   0},
  { S_BreburnWT,    UECSINPUTDATA,  U_sec,  D_reburn,    DUMMY,       0,        &(W_BreburnWT),   0,     3600,   0},  
  { S_BstopT,       UECSSHOWDATA,   U_sec,  D_reburn,    DUMMY,       0,        &(W_BstopT),      0,    86400,   0},  
  { S_BpreCO2Tsr,   UECSINPUTDATA,  U_sec,  D_pre,       DUMMY,       0,        &(W_BpreCO2Tsr),  0,     7200,   0},  
  { S_darkKeepT,    UECSSHOWDATA,   U_sec,  D_darkKeep,  DUMMY,       0,        &(W_darkKeepT),   0,    86400,   0},  
  { S_CO2vopUse,    UECSSELECTDATA, EMPTY,  D_vopUse,    SS_CO2vopUse,2,        &(W_CO2vopUse),   0,      1,     0},  
  { S_CO2tempOpen,  UECSINPUTDATA,  U_temp, D_upLow,     DUMMY,       0,        &(W_CO2tempOpen), 0,     40,     0},  
  { S_CO2tempPhsy,  UECSINPUTDATA,  U_temp, D_downLow,   DUMMY,       0,        &(W_CO2tempPhsy), 0,     40,     0},
  { S_CO2solarPhsy, UECSINPUTDATA,  U_flux, D_downLow,   DUMMY,       0,        &(W_CO2solarPhsy),0,     2000,   3}
} ;

//////////////////////////////////
// UserCCM setting
//////////////////////////////////

//define CCMID for identify
//CCMID_dummy must put on last
enum { 
  CCMID_InAirTemp_mIC, CCMID_InAirCO2_mIC,  CCMID_WRadiation_mOC,
  CCMID_OpenP_cRV,
  CCMID_Time,          CCMID_cnd_cCO,       CCMID_CO2supTime_cCO,
  CCMID_CO2supBTime_cCO,  CCMID_CO2SP_cCO,  CCMID_esec_cCO,
  CCMID_dummy
};
const int U_MAX_CCM = CCMID_dummy;
UECSCCM U_ccmList[U_MAX_CCM];

const char ccmNameInAirTemp[] PROGMEM = "室内気温";
const char ccmTypeInAirTemp[] PROGMEM = "InAirTemp.mIC";
const char ccmUnitInAirTemp[] PROGMEM = "C";

const char ccmNameInAirCO2[] PROGMEM = "CO2濃度";
const char ccmTypeInAirCO2[] PROGMEM = "InAirCO2.mIC";
const char ccmUnitInAirCO2[] PROGMEM = "ppm";

const char ccmNameWRadiation[] PROGMEM = "屋外日射";
const char ccmTypeWRadiation[] PROGMEM = "WRadiation.mOC";
const char ccmUnitWRadiation[] PROGMEM = "kW/m2";

const char ccmNameOpenP[] PROGMEM = "換気窓開度";
const char ccmTypeOpenP[] PROGMEM = "OpenP.cRV";
const char ccmUnitOpenP[] PROGMEM = "%";

const char ccmNameTime[] PROGMEM = "時刻";
const char ccmTypeTime[] PROGMEM = "Time";
const char ccmUnitTime[] PROGMEM = "hhmmss";

const char ccmNamecnd[] PROGMEM = "状態";
const char ccmTypecnd[] PROGMEM = "cnd.cCO";
const char ccmUnitcnd[] PROGMEM = "";

const char ccmNameCO2supTime[] PROGMEM = "CO2ボンベ供給時間";
const char ccmTypeCO2supTime[] PROGMEM = "CO2supTime.cCO";
const char ccmUnitCO2supTime[] PROGMEM = "s/d-1";

const char ccmNameCO2supBTime[] PROGMEM = "CO2燃焼型供給時間";
const char ccmTypeCO2supBTime[] PROGMEM = "CO2supBTime.cCO";
const char ccmUnitCO2supBTime[] PROGMEM = "s/d-1";

const char ccmNameCO2SP[] PROGMEM = "CO2供給濃度";
const char ccmTypeCO2SP[] PROGMEM = "CO2SP.cCO";
const char ccmUnitCO2SP[] PROGMEM = "ppm";

// WDT動作確認用
const char ccmNameEsec[] PROGMEM = "経過時間";
const char ccmTypeEsec[] PROGMEM = "esec.cCO";
const char ccmUnitsec[] PROGMEM = "s";

//------------------------------------------------------
//UARDECS初期化用関数
//主にCCMの作成とMACアドレスの設定を行う
//------------------------------------------------------
void UserInit(){

 //Set ccm list
 //         send,  CCMID,                Name,              Type Name,         Unit,              Pri, dig, class
 UECSsetCCM(false, CCMID_InAirTemp_mIC,  ccmNameInAirTemp,  ccmTypeInAirTemp,  ccmUnitInAirTemp,  29,  1,   A_10S_0 );
 UECSsetCCM(false, CCMID_InAirCO2_mIC,   ccmNameInAirCO2,   ccmTypeInAirCO2,   ccmUnitInAirCO2,   29,  0,   A_10S_0 );
 UECSsetCCM(false, CCMID_WRadiation_mOC, ccmNameWRadiation, ccmTypeWRadiation, ccmUnitWRadiation, 29,  3,   A_10S_0 );
 UECSsetCCM(false, CCMID_OpenP_cRV,      ccmNameOpenP,      ccmTypeOpenP,      ccmUnitOpenP,      29,  0,   A_10S_0 );
 UECSsetCCM(false, CCMID_Time,           ccmNameTime,       ccmTypeTime,       ccmUnitTime,       29,  0,   A_1M_0  );
 UECSsetCCM(true,  CCMID_cnd_cCO,        ccmNamecnd,        ccmTypecnd,        ccmUnitcnd,        30,  0,   A_1S_0  );
 UECSsetCCM(true,  CCMID_CO2supTime_cCO, ccmNameCO2supTime, ccmTypeCO2supTime, ccmUnitCO2supTime, 30,  0,   A_10S_0 );
 UECSsetCCM(true,  CCMID_CO2supBTime_cCO,ccmNameCO2supBTime,ccmTypeCO2supBTime,ccmUnitCO2supBTime,30,  0,   A_10S_0 );
 UECSsetCCM(true,  CCMID_CO2SP_cCO,      ccmNameCO2SP,      ccmTypeCO2SP,      ccmUnitCO2SP,      30,  0,   A_10S_0 );
 UECSsetCCM(true,  CCMID_esec_cCO,       ccmNameEsec,       ccmTypeEsec,       ccmUnitsec,        30,  0,   A_1S_0  );  // WDT動作確認用

 //Set proper MAC address
 U_orgAttribute.mac[0] = mac0;
 U_orgAttribute.mac[1] = mac1;
 U_orgAttribute.mac[2] = mac2;
 U_orgAttribute.mac[3] = mac3;
 U_orgAttribute.mac[4] = mac4;
 U_orgAttribute.mac[5] = mac5;

}

//---------------------------------------------------------
//起動直後に１回呼び出される関数。
//様々な初期化処理を記述できる。
//この関数内ではUECSsetup()関数を呼び出さなくてはならない。
//必要に応じて処理を記述してもかまわない。
//---------------------------------------------------------
void setup(){

  //制御シールドの制御信号線の初期化
  pinMode(in_out, OUTPUT) ;
  digitalWrite(in_out, LOW) ;
  for(i = 0; i < 4; i++) {
    pinMode(port[i], OUTPUT) ;
    digitalWrite(port[i], LOW) ;
  }
  pinMode(ratch, OUTPUT) ;
  digitalWrite(ratch, HIGH) ;
  delayMicroseconds(1) ; 
  digitalWrite(ratch, LOW) ;
  pinMode(wdt, OUTPUT) ;
  
  digitalWrite(wdt, HIGH) ; // ウオッチドッグクリア信号を初期値にする
  clearWdt() ;
  clearDout() ; // 出力をリセット

  UECSsetup();
  
  // W_mode変数の異常でリセットの条件判断
  if((W_Chmode<0)||(W_Chmode>7)) {
    W_Chmode = 7 ; //全値リセットに設定
    initializeSetpoints();
    }
  // 内部時刻のリセット
  innerTime = 0 ;
  secCount = 60 ;
  // CO2作業変数をリセット
  CO2workTime = 0 ;
  SrPreCO2T = 0 ;
  W_BstopT = 0 ; // 消火後経過時間
  W_darkKeepT = 0 ; // 暗黒継続時間

}

//------------------------------------------------------
// EPROM関係の制御設定値等をクリアする。
// W_modeの変数値がおかしい場合もリセット
//------------------------------------------------------
void initializeSetpoints() {
  W_CO2cond  = 0 ; // CO2制御状態 生ガス
  W_DCO2supTime = 0 ; // 昨日CO2供給
  W_CO2condB  = 0 ; // CO2制御状態　燃焼
  W_DCO2supBTime = 0 ; // 昨日CO2供給
  W_Chmode = 0 ; // 自動モード
  W_CO2highSP = 1000 ; // 設定CO2濃度
  W_CO2injsec = 15 ;  // CO2注入時間
  W_CO2postsec = 300 ; // CO2後待時間
  W_BCO2HhighSP = 1250 ; // 設定CO2濃度
  W_BCO2HlowSP = 750 ; // 設定CO2濃度
  W_CO2LowSP = 400 ;  // 換気時CO2設定濃度
  W_BreburnWT = 1800 ; // 再燃焼待ち時間
  W_BstopT = 0 ; // 消火後経過時間
  W_BpreCO2Tsr = 1800 ; // 夜明け時のプリ施用時間
  W_darkKeepT = 0 ; // 暗黒継続時間
  W_CO2vopUse = 0 ;// 窓開度使用しない
  W_CO2tempOpen = 27 ;// CO2上限気温
  W_CO2tempPhsy = 10 ; // CO2下限気温
  W_CO2solarPhsy = 150 ; // CO2下限日射

  U_ccmList[CCMID_CO2supTime_cCO].value = 0 ; // CO2供給時間クリア
  U_ccmList[CCMID_CO2supBTime_cCO].value = 0 ; // CO2供給時間クリア
}

//------------------------------------------------------
// 時刻の処理(1分毎に呼び出す)読めなかった時には内部時計を1分進める
//------------------------------------------------------
void checkTime1min() {
  if(!U_ccmList[CCMID_Time].validity) {
    // UECS-CCMが正しく受信できない場合、内部時計を1分増やす
    if((innerTime % 100) >= 59) { // 59分なら時間に1加える
      innerTime = (innerTime / 100 + 1) * 100 ;
      if(innerTime >= 2400) innerTime = 0 ; // 24時は0時に
    }
    else innerTime++ ; // それ以外は1分加える
  }
}
//------------------------------------------------------
// 時刻の読込(1秒毎に呼び出す)
//------------------------------------------------------
void checkTime1sec() {
  if(U_ccmList[CCMID_Time].validity) {
    innerTime = U_ccmList[CCMID_Time].value / 100 ; // 秒の単位を落として内部時計に代入
  }
}

//------------------------------------------------------
// CO2供給設定濃度の計算(CCM変数に代入)
// 毎秒呼出
//------------------------------------------------------
void calcCO2SP() {
  int workSP ;
  workSP = 0 ; // 初期値0ppm
  if((W_Chmode == 1)||(W_Chmode == 2)) {
    workSP = (int)W_CO2LowSP ; // 自動の時の暗黙値(低濃度)にする。
    if(U_ccmList[CCMID_WRadiation_mOC].validity) {
      if(U_ccmList[CCMID_WRadiation_mOC].value >= W_CO2solarPhsy) {
      //日射フラックスが一定以上ある
        if(U_ccmList[CCMID_InAirTemp_mIC].validity) {
          if(U_ccmList[CCMID_InAirTemp_mIC].value >= (W_CO2tempPhsy * 10)) {
          // 下限気温以上
            if(W_CO2vopUse == 0) { // 換気窓開度は考慮しない
              if(U_ccmList[CCMID_InAirTemp_mIC].value < (W_CO2tempOpen * 10)) {
              // 上限気温未満
                if(W_Chmode == 1) workSP = W_CO2highSP ;
                //液化CO2施用自動時
                if(W_Chmode == 2) workSP = ( W_BCO2HhighSP + W_BCO2HlowSP ) / 2 ;
                // 燃焼式CO2施用自動時
              }
            }
            else if(W_CO2vopUse == 1) { // 換気窓開度を考慮する
              if(U_ccmList[CCMID_OpenP_cRV].validity) {
                if(U_ccmList[CCMID_OpenP_cRV].value == 0) {
                // 換気窓開度0%
                  if(W_Chmode == 1) workSP = W_CO2highSP ;
                  //液化CO2施用自動時
                  if(W_Chmode == 2) workSP = ( W_BCO2HhighSP + W_BCO2HlowSP ) / 2 ;
                  // 燃焼式CO2施用自動時
                }
              }
            }
          }
        }
      }
    }
  }
  U_ccmList[CCMID_CO2SP_cCO].value = workSP ;
}
  
//------------------------------------------------------
// 状態とリセット後の秒数EsecをCCM送出のためセット
// 毎秒チェック
//------------------------------------------------------
void setCCM(){
  // 制御稼働時間設定
  if(CO2_supply == HIGH)  U_ccmList[CCMID_CO2supTime_cCO].value += 1 ; // CO2 生ガス
  if(CO2B_supply == HIGH)  U_ccmList[CCMID_CO2supBTime_cCO].value += 1 ; // CO2 燃焼

  // cndに制御モード値をセット
  U_ccmList[CCMID_cnd_cCO].value = W_Chmode ;

  U_ccmList[CCMID_esec_cCO].value = (signed long)(millis() / 1000) ; // WDT動作確認用
}

//------------------------------------------------------
// 液化CO2自動制御
// 毎秒呼出
//------------------------------------------------------
void CO2auto() {
  // 自動か強制のCO2供給モードのときだけ実行
  if((W_Chmode == 1)||(W_Chmode == 4)) {
     if(CO2workTime > 0) { // CO2供給動作中処理
       FAN_operate = HIGH ;
       W_CO2cond = 1 ; // 前送風
       if((CO2workTime > C_CO2presec)&&(CO2workTime <= (C_CO2presec + W_CO2injsec))) {
         // CO2注入タイミングか?
         CO2_supply = HIGH ; // CO2注入時間帯だけON
         W_CO2cond = 2 ; // 供給
       }
       else {
         CO2_supply = LOW ; // 注入停止
       }
       if(CO2workTime > (C_CO2presec + W_CO2injsec)) { // 拡散待ちか
         W_CO2cond = 3 ; // 後送風
       }
       if(CO2workTime > (C_CO2presec + W_CO2injsec + W_CO2postsec)) { // 供給サイクル完了
         CO2workTime = 0 ;
         FAN_operate = LOW ;
         W_CO2cond = 0 ; // 停止
       }
       else CO2workTime++ ; // 完了でなければ1秒進める
     }
     else { // CO2を供給していないときには濃度チェック
       if(W_Chmode == 4) { // 強制動作モードのままならば、供給サイクルを再起動
           FAN_operate = HIGH ;
           CO2workTime = 1 ;
           W_CO2cond = 1 ; // 前送風
       }
       else {
         if(U_ccmList[CCMID_InAirCO2_mIC].validity) {  // CCMが有効で設定濃度を下回っていれば供給
           if(U_ccmList[CCMID_InAirCO2_mIC].value < U_ccmList[CCMID_CO2SP_cCO].value) {
             FAN_operate = HIGH ;
             CO2workTime = 1 ;
             W_CO2cond = 1 ; // 前送風
           }
         }
       }
     }
  }
}

//------------------------------------------------------
// 燃焼CO2自動制御
// 毎秒呼出
//------------------------------------------------------
void BurnCO2auto() {
  
  // 燃焼式特有の暗黒継続時間と低継続時間をカウント
  if(U_ccmList[CCMID_WRadiation_mOC].validity) {
    if(U_ccmList[CCMID_WRadiation_mOC].value < 1) {
    // 日射量0.001未満で暗黒カウント 上限1日分の秒
      if(++W_darkKeepT > 86400) W_darkKeepT = 86400 ;
    }
    else {
      if((W_Chmode == 2)&&(W_darkKeepT > C_minDarkKeepT)&&(W_BpreCO2Tsr > 0)) {
        // 燃焼モードの時、一定時間の暗黒の後明るくなって、プリ施用が設定されていたら
        W_CO2condB = 2 ; // 夜明けプリ施用モードに設定
        SrPreCO2T = 0 ; // プリ供給カウンタリセット
      }
      W_darkKeepT = 0 ; // 暗黒継続時間リセット
    }
  }

  // 自動の燃焼式CO2供給モードのときだけ実行
  if(W_Chmode == 2) {
    if(W_CO2condB == 2) { // 夜明けプリ施用モードだったら 
      //夜明プリ施用
      FAN_operate = HIGH ;
      CO2B_supply = HIGH ; // 燃焼式装置稼働
      SrPreCO2T++ ; // プリ供給カウンタ進める
    }
    // プリ供給時間経過で停止
    if((W_CO2condB == 2)&&(SrPreCO2T > W_BpreCO2Tsr)) {
      FAN_operate = LOW ;
      CO2B_supply = LOW ; // 燃焼式装置停止
      W_CO2condB = 0 ; // 夜明けプリモード離脱
      SrPreCO2T = 0 ; // プリ供給カウンタリセット
      W_BstopT = 0 ; // 停止継続時間リセット
    }

    // 停止継続時間をカウント  上限1日分の秒
    if(CO2B_supply == LOW) {
      if(++W_BstopT > 86400) W_BstopT = 86400 ;
    }
    else W_BstopT = 0 ;
        
    // 通常施用条件テスト
    if((SrPreCO2T == 0)&&(W_BstopT > W_BreburnWT)) {
      // 夜明けプリ施用しておらず、再燃焼待ち時間を経過した
      if(U_ccmList[CCMID_WRadiation_mOC].validity) {
        if(U_ccmList[CCMID_WRadiation_mOC].value > 0) {
        // 日射が0ではない(夜間に無駄に施用してしまう対策用)
          if(U_ccmList[CCMID_InAirCO2_mIC].validity) {  // CCMが有効で設定濃度を下回っていれば供給
            if(U_ccmList[CCMID_CO2SP_cCO].value == W_CO2LowSP) { 
            // 低濃度施用かどうか
              if(U_ccmList[CCMID_InAirCO2_mIC].value < W_CO2LowSP) {
              // 低濃度施用
                W_CO2condB = 1 ;
                FAN_operate = HIGH ;
                CO2B_supply = HIGH ; // 燃焼式装置稼働
              }
            }
            else {
              if(U_ccmList[CCMID_InAirCO2_mIC].value < W_BCO2HlowSP) {
                // 高濃度施用
                W_CO2condB = 1 ;
                FAN_operate = HIGH ;
                CO2B_supply = HIGH ; // 燃焼式装置稼働
              }
            }
          }
        }
      }
    }
      
    // 停止条件テスト
    if(W_CO2condB == 1) {
      if(U_ccmList[CCMID_InAirCO2_mIC].validity) {  // CCMが有効
        if(U_ccmList[CCMID_CO2SP_cCO].value == W_CO2LowSP) { 
          // 低濃度施用かどうか
          if(U_ccmList[CCMID_InAirCO2_mIC].value > W_CO2LowSP + 50) {
            // 低濃度施用停止 ヒステリシス+50ppm
            W_CO2condB = 0 ;
            FAN_operate = LOW ;
            CO2B_supply = LOW ; // 燃焼式装置稼働
            W_BstopT = 0 ; // 停止継続時間リセット
          }
        }
        else {
          if(U_ccmList[CCMID_InAirCO2_mIC].value > W_BCO2HhighSP) {
            // 高濃度施用停止
            W_CO2condB = 0 ;
            FAN_operate = LOW ;
            CO2B_supply = LOW ; // 燃焼式装置稼働
            W_BstopT = 0 ; // 停止継続時間リセット
          }
        }
      }
    }
  }
}
         
//---------------------------------------------------------
//Webページから入力が行われ各種値を設定し不揮発性メモリに値を保存後、
//ページの再描画を行う前に以下の関数が呼び出される。
//---------------------------------------------------------
void OnWebFormRecieved(){
  
  // リセットする場合の処理
  if(W_Chmode == 6) {
    initializeSetpoints() ;
    // 内部時刻のリセット
    innerTime = 0 ;
    secCount = 60 ;
    // CO2作業変数をリセット
    CO2workTime = 0 ;
    SrPreCO2T = 0 ;
  }
  
  // 強制停止処理
  if(W_Chmode == 0) {
    clearDout() ; // 全ての制御出力OFF
    CO2workTime = 0 ; // CO2供給強制中止
    SrPreCO2T = 0 ; // 燃焼夜明プリ施用停止
    W_CO2cond = 0 ; // 液化CO2供給状態を停止
    W_CO2condB  = 0 ; // 燃焼CO2制御状態を停止
    W_BstopT = 0 ; // 停止継続時間リセット
  }
  // 液化CO2自動施用モード
  if(W_Chmode == 1) {
    CO2B_supply = LOW ;
    SrPreCO2T = 0 ; // 燃焼夜明プリ施用停止
    W_CO2condB  = 0 ; // 燃焼CO2制御状態を停止
  }
  // 燃焼CO2自動施用モード
  if(W_Chmode == 2) {
    CO2_supply = LOW ;
    CO2workTime = 0 ; // CO2供給強制中止
    W_CO2cond = 0 ; // 液化CO2供給状態を停止
  }
  // 強制送風動作
  if(W_Chmode == 3) {
    CO2_supply = CO2B_supply = LOW ;
    FAN_operate = HIGH ;
    CO2workTime = 0 ; // CO2供給強制中止
    SrPreCO2T = 0 ; // 燃焼夜明プリ施用停止
    W_CO2cond = 0 ; // CO2供給状態を停止
    W_CO2condB  = 0 ; // 燃焼CO2制御状態を停止
  }
  // 強制液化CO2供給動作
  if(W_Chmode == 4) {
    CO2_supply = CO2B_supply = LOW ;
    FAN_operate = HIGH ;
    SrPreCO2T = 0 ; // 燃焼夜明プリ施用停止
    W_CO2condB  = 0 ; // 燃焼CO2制御状態を停止
    CO2workTime = 1 ; // CO2供給を開始に
    W_CO2cond = 1 ; // CO2供給状態を前送風に
  }
  // 強制燃焼CO2供給動作
  if(W_Chmode == 5) {
    CO2_supply =  LOW ;
    CO2B_supply = FAN_operate = HIGH ;
    SrPreCO2T = 0 ; // 燃焼夜明プリ施用停止
    W_CO2condB  = 1 ; // 燃焼CO2制御状態を開始に
    W_BstopT = 0 ; // 停止継続時間リセット
    CO2workTime = 0 ; // CO2供給を停止に
    W_CO2cond = 0 ; // CO2供給状態を停止に
  }

}

//---------------------------------------------------------
//１秒に１回呼び出される関数
//---------------------------------------------------------
void UserEverySecond(){

  //ウオッチドッグタイマークリア
  clearWdt() ;
  checkTime1sec() ; // 時間をUPDATE
  
  if(secCount >= 60) {
    //////////////////////////////////////////////////////
    // 1分毎の処理
    // void UserEveryMinute()は誤差がある可能性があるので
    // 使用しない(1回給液しなかったりすると大変!)
    //////////////////////////////////////////////////////
    secCount = 0 ; // 秒カウンターをクリア
    checkTime1min() ;
    
    
    // 日界処理　= 0:00の処理
    if(innerTime == 0) {
      // 液化CO2関係
      W_DCO2supTime = U_ccmList[CCMID_CO2supTime_cCO].value ; //昨日CO2供給時間セット
      U_ccmList[CCMID_CO2supTime_cCO].value = 0 ; // 日CO2供給時間クリア
      // 燃焼CO2関係
      W_DCO2supBTime = U_ccmList[CCMID_CO2supBTime_cCO].value ; //昨日CO2供給時間セット
      U_ccmList[CCMID_CO2supBTime_cCO].value = 0 ; // 日CO2供給時間クリア
    }
  }
  
  /////////////////////
  // 以下は1秒毎の処理
  /////////////////////
  calcCO2SP() ; // CO2設定値計算
  CO2auto() ; // 液化CO2自動制御
  BurnCO2auto() ; // 燃焼CO2自動制御
  controlOut() ; // 制御信号出力
  setCCM() ; // CCMに値セット
  secCount++ ; // 秒カウンターを1up
  
}

//---------------------------------------------------------
//１分に１回呼び出される関数
//---------------------------------------------------------
void UserEveryMinute(){
}

//---------------------------------------------------------
//メインループ
//システムのタイマカウント，httpサーバーの処理，
//UDP16520番ポートと16529番ポートの通信文をチェックした後，呼び出さされる関数。
//呼び出される頻度が高いため，重い処理を記述しないこと。
//---------------------------------------------------------
void UserEveryLoop(){
}

//---------------------------------------------------------
//setup()実行後に呼び出されるメインループ
//この関数内ではUECSloop()関数を呼び出さなくてはならない。
//UserEveryLoop()に似ているがネットワーク関係の処理を行う前に呼び出される。
//必要に応じて処理を記述してもかまわない。
//呼び出される頻度が高いため,重い処理を記述しないこと。
//---------------------------------------------------------
void loop(){
  UECSloop();
}
